/**
 * STAMP API
 * Neopost STAMP API. This API provides Shipping Tracking Accounting and Mailing functionalities.
 *
 * OpenAPI spec version: 1.0.0
 * 
 *
 * NOTE: This class is auto generated by the swagger code generator program.
 * https://github.com/swagger-api/swagger-codegen.git
 * Do not edit the class manually.
 */

import { Inject, Injectable, Optional }                      from '@angular/core';
import { Http, Headers, URLSearchParams }                    from '@angular/http';
import { RequestMethod, RequestOptions, RequestOptionsArgs } from '@angular/http';
import { Response, ResponseContentType }                     from '@angular/http';

import { Observable }                                        from 'rxjs/Observable';
import 'rxjs/add/operator/map';

import * as models                                           from '../model/models';
import { BASE_PATH }                                         from '../variables';
import { Configuration }                                     from '../configuration';

/* tslint:disable:no-unused-variable member-ordering */


@Injectable()
export class CarrierApi {
    protected basePath = 'https://api-dev.neopost.com/stamp/v1';
    public defaultHeaders: Headers = new Headers();
    public configuration: Configuration = new Configuration();

    constructor(protected http: Http, @Optional()@Inject(BASE_PATH) basePath: string, @Optional() configuration: Configuration) {
        if (basePath) {
            this.basePath = basePath;
        }
        if (configuration) {
            this.configuration = configuration;
        }
    }

    /**
     * 
     * Extends object by coping non-existing properties.
     * @param objA object to be extended
     * @param objB source object
     */
    private extendObj<T1,T2>(objA: T1, objB: T2) {
        for(let key in objB){
            if(objB.hasOwnProperty(key)){
                (objA as any)[key] = (objB as any)[key];
            }
        }
        return <T1&T2>objA;
    }

    /**
     * Get the list of carrier-specific packaging
     * 
     * @param carrier Carrier code
     * @param service service code
     */
    public getCarrierPackaging(carrier: string, service?: string, extraHttpRequestParams?: any): Observable<models.PackagingList> {
        return this.getCarrierPackagingWithHttpInfo(carrier, service, extraHttpRequestParams)
            .map((response: Response) => {
                if (response.status === 204) {
                    return undefined;
                } else {
                    return response.json();
                }
            });
    }

    /**
     * Get the list of carriers
     * 
     */
    public getCarriers(extraHttpRequestParams?: any): Observable<models.CarrierList> {
        return this.getCarriersWithHttpInfo(extraHttpRequestParams)
            .map((response: Response) => {
                if (response.status === 204) {
                    return undefined;
                } else {
                    return response.json();
                }
            });
    }

    /**
     * Get the list of carrier rules
     * 
     * @param carrier Carrier code
     * @param service Service code
     */
    public getRules(carrier: string, service: string, extraHttpRequestParams?: any): Observable<models.RulesResponse> {
        return this.getRulesWithHttpInfo(carrier, service, extraHttpRequestParams)
            .map((response: Response) => {
                if (response.status === 204) {
                    return undefined;
                } else {
                    return response.json();
                }
            });
    }

    /**
     * Get the list of carrier service options
     * 
     * @param carrier carrier code
     * @param service service code
     */
    public getServiceOptions(carrier: string, service: string, extraHttpRequestParams?: any): Observable<models.Service> {
        return this.getServiceOptionsWithHttpInfo(carrier, service, extraHttpRequestParams)
            .map((response: Response) => {
                if (response.status === 204) {
                    return undefined;
                } else {
                    return response.json();
                }
            });
    }

    /**
     * Get the list of carrier services
     * 
     * @param carrier Carrier code
     * @param destination Destination type
     */
    public getServices(carrier: string, destination?: string, extraHttpRequestParams?: any): Observable<models.ServicesResponse> {
        return this.getServicesWithHttpInfo(carrier, destination, extraHttpRequestParams)
            .map((response: Response) => {
                if (response.status === 204) {
                    return undefined;
                } else {
                    return response.json();
                }
            });
    }


    /**
     * Get the list of carrier-specific packaging
     * 
     * @param carrier Carrier code
     * @param service service code
     */
    public getCarrierPackagingWithHttpInfo(carrier: string, service?: string, extraHttpRequestParams?: any): Observable<Response> {
        const path = this.basePath + `/carriers/${carrier}/packaging`;

        let queryParameters = new URLSearchParams();
        let headers = new Headers(this.defaultHeaders.toJSON()); // https://github.com/angular/angular/issues/6845
        // verify required parameter 'carrier' is not null or undefined
        if (carrier === null || carrier === undefined) {
            throw new Error('Required parameter carrier was null or undefined when calling getCarrierPackaging.');
        }
        if (service !== undefined) {
            queryParameters.set('service', <any>service);
        }


        // to determine the Content-Type header
        let consumes: string[] = [
            'text/plain'
        ];

        // to determine the Accept header
        let produces: string[] = [
            'application/json'
        ];
        
        // authentication (jwtAuth) required
        // oauth required
        if (this.configuration.accessToken)
        {
            let accessToken = typeof this.configuration.accessToken === 'function'
                ? this.configuration.accessToken()
                : this.configuration.accessToken;
            headers.set('Authorization', 'Bearer ' + accessToken);
        }
            



        let requestOptions: RequestOptionsArgs = new RequestOptions({
            method: RequestMethod.Get,
            headers: headers,
            search: queryParameters
        });
        
        // https://github.com/swagger-api/swagger-codegen/issues/4037
        if (extraHttpRequestParams) {
            requestOptions = this.extendObj(requestOptions, extraHttpRequestParams);
        }

        return this.http.request(path, requestOptions);
    }

    /**
     * Get the list of carriers
     * 
     */
    public getCarriersWithHttpInfo(extraHttpRequestParams?: any): Observable<Response> {
        const path = this.basePath + `/carriers`;

        let queryParameters = new URLSearchParams();
        let headers = new Headers(this.defaultHeaders.toJSON()); // https://github.com/angular/angular/issues/6845


        // to determine the Content-Type header
        let consumes: string[] = [
            'text/plain'
        ];

        // to determine the Accept header
        let produces: string[] = [
            'application/json'
        ];
        
        // authentication (jwtAuth) required
        // oauth required
        if (this.configuration.accessToken)
        {
            let accessToken = typeof this.configuration.accessToken === 'function'
                ? this.configuration.accessToken()
                : this.configuration.accessToken;
            headers.set('Authorization', 'Bearer ' + accessToken);
        }
            



        let requestOptions: RequestOptionsArgs = new RequestOptions({
            method: RequestMethod.Get,
            headers: headers,
            search: queryParameters
        });
        
        // https://github.com/swagger-api/swagger-codegen/issues/4037
        if (extraHttpRequestParams) {
            requestOptions = this.extendObj(requestOptions, extraHttpRequestParams);
        }

        return this.http.request(path, requestOptions);
    }

    /**
     * Get the list of carrier rules
     * 
     * @param carrier Carrier code
     * @param service Service code
     */
    public getRulesWithHttpInfo(carrier: string, service: string, extraHttpRequestParams?: any): Observable<Response> {
        const path = this.basePath + `/carriers/${carrier}/rules`;

        let queryParameters = new URLSearchParams();
        let headers = new Headers(this.defaultHeaders.toJSON()); // https://github.com/angular/angular/issues/6845
        // verify required parameter 'carrier' is not null or undefined
        if (carrier === null || carrier === undefined) {
            throw new Error('Required parameter carrier was null or undefined when calling getRules.');
        }
        // verify required parameter 'service' is not null or undefined
        if (service === null || service === undefined) {
            throw new Error('Required parameter service was null or undefined when calling getRules.');
        }
        if (service !== undefined) {
            queryParameters.set('service', <any>service);
        }


        // to determine the Content-Type header
        let consumes: string[] = [
            'text/plain'
        ];

        // to determine the Accept header
        let produces: string[] = [
            'application/json'
        ];
        
        // authentication (jwtAuth) required
        // oauth required
        if (this.configuration.accessToken)
        {
            let accessToken = typeof this.configuration.accessToken === 'function'
                ? this.configuration.accessToken()
                : this.configuration.accessToken;
            headers.set('Authorization', 'Bearer ' + accessToken);
        }
            



        let requestOptions: RequestOptionsArgs = new RequestOptions({
            method: RequestMethod.Get,
            headers: headers,
            search: queryParameters
        });
        
        // https://github.com/swagger-api/swagger-codegen/issues/4037
        if (extraHttpRequestParams) {
            requestOptions = this.extendObj(requestOptions, extraHttpRequestParams);
        }

        return this.http.request(path, requestOptions);
    }

    /**
     * Get the list of carrier service options
     * 
     * @param carrier carrier code
     * @param service service code
     */
    public getServiceOptionsWithHttpInfo(carrier: string, service: string, extraHttpRequestParams?: any): Observable<Response> {
        const path = this.basePath + `/carriers/${carrier}/options`;

        let queryParameters = new URLSearchParams();
        let headers = new Headers(this.defaultHeaders.toJSON()); // https://github.com/angular/angular/issues/6845
        // verify required parameter 'carrier' is not null or undefined
        if (carrier === null || carrier === undefined) {
            throw new Error('Required parameter carrier was null or undefined when calling getServiceOptions.');
        }
        // verify required parameter 'service' is not null or undefined
        if (service === null || service === undefined) {
            throw new Error('Required parameter service was null or undefined when calling getServiceOptions.');
        }
        if (service !== undefined) {
            queryParameters.set('service', <any>service);
        }


        // to determine the Content-Type header
        let consumes: string[] = [
            'text/plain'
        ];

        // to determine the Accept header
        let produces: string[] = [
            'application/json'
        ];
        
        // authentication (jwtAuth) required
        // oauth required
        if (this.configuration.accessToken)
        {
            let accessToken = typeof this.configuration.accessToken === 'function'
                ? this.configuration.accessToken()
                : this.configuration.accessToken;
            headers.set('Authorization', 'Bearer ' + accessToken);
        }
            



        let requestOptions: RequestOptionsArgs = new RequestOptions({
            method: RequestMethod.Get,
            headers: headers,
            search: queryParameters
        });
        
        // https://github.com/swagger-api/swagger-codegen/issues/4037
        if (extraHttpRequestParams) {
            requestOptions = this.extendObj(requestOptions, extraHttpRequestParams);
        }

        return this.http.request(path, requestOptions);
    }

    /**
     * Get the list of carrier services
     * 
     * @param carrier Carrier code
     * @param destination Destination type
     */
    public getServicesWithHttpInfo(carrier: string, destination?: string, extraHttpRequestParams?: any): Observable<Response> {
        const path = this.basePath + `/carriers/${carrier}/services`;

        let queryParameters = new URLSearchParams();
        let headers = new Headers(this.defaultHeaders.toJSON()); // https://github.com/angular/angular/issues/6845
        // verify required parameter 'carrier' is not null or undefined
        if (carrier === null || carrier === undefined) {
            throw new Error('Required parameter carrier was null or undefined when calling getServices.');
        }
        if (destination !== undefined) {
            queryParameters.set('destination', <any>destination);
        }


        // to determine the Content-Type header
        let consumes: string[] = [
            'text/plain'
        ];

        // to determine the Accept header
        let produces: string[] = [
            'application/json'
        ];
        
        // authentication (jwtAuth) required
        // oauth required
        if (this.configuration.accessToken)
        {
            let accessToken = typeof this.configuration.accessToken === 'function'
                ? this.configuration.accessToken()
                : this.configuration.accessToken;
            headers.set('Authorization', 'Bearer ' + accessToken);
        }
            



        let requestOptions: RequestOptionsArgs = new RequestOptions({
            method: RequestMethod.Get,
            headers: headers,
            search: queryParameters
        });
        
        // https://github.com/swagger-api/swagger-codegen/issues/4037
        if (extraHttpRequestParams) {
            requestOptions = this.extendObj(requestOptions, extraHttpRequestParams);
        }

        return this.http.request(path, requestOptions);
    }

}
